import json
import time
from pathlib import Path

import flask
import requests

from secret import token

REVERSE_PROXY = False # if True: uses X-Real-IP header instead of remote addr
BASE_DIR = Path(__file__).parent
TIMEOUT = 15 * 60
MAX_LENGTH = 100_000
CONTENT_WARNING = "Post can contain any text"
PROMO_URL = "\n\nhttps://everyone.trinkey.com/"
POST_URL = "https://is.trinkey.com/api/iceshrimp/notes"
RUN_CONF = {
    "debug": True,
    "host": "0.0.0.0",
    "port": "8000"
}

app = flask.Flask(__file__)

data: dict[str, int] = {}

try:
    data = json.load(open(BASE_DIR / "blocked.json", "r"))
except FileNotFoundError:
    ...
except json.JSONDecodeError:
    ...

def save():
    now = time.time()

    to_remove = []
    for i in data:
        if data[i] < now:
            to_remove.append(i)

    for i in to_remove:
        del data[i]

    with open(BASE_DIR / "blocked.json", "w") as f:
        json.dump(data, f)

def get_ip() -> str:
    ip = (flask.request.headers.get("X-Real-IP") if REVERSE_PROXY else flask.request.remote_addr) or "0.0.0.0"

    if ":" in ip:
        ip = ip[:20] + ":/64"

    return ip

@app.route("/", methods=["POST", "GET"])
def index() -> bytes:
    ip = get_ip()
    cant_post = ip in data and data[ip] > time.time()
    message = ""

    if flask.request.method == "POST":
        if cant_post:
            message = "You can't post yet!"
        else:
            content = (flask.request.form.get("text") or "").strip()[:MAX_LENGTH - len(PROMO_URL)]

            if content:
                resp = requests.post(POST_URL, json={
                    "text": content + PROMO_URL,
                    "cw": CONTENT_WARNING,
                    "replyId": None,
                    "renoteId": None,
                    "mediaIds": None,
                    "visibility": "home",
                    "idempotencyKey": None
                }, headers={
                    "Authorization": f"Bearer {token}"
                })

                if resp.status_code == 200:
                    cant_post = True
                    data[ip] = int(time.time() + TIMEOUT)
                    save()
                    message = "Success!"
                else:
                    message = f"Got non-200 status code ({resp.status_code})"
            else:
                message = "Enter some text!"

    return open(BASE_DIR / "public/index.html", "rb").read() \
            .replace(b"{{ IP }}", str.encode(ip)) \
            .replace(b"{{ EXPIRE }}", str.encode(str(data[ip] if cant_post else 0))) \
            .replace(b"{{ ERROR }}", str.encode(message)) \
            .replace(b"{{ MAX_LENGTH }}", str.encode(str(MAX_LENGTH - len(PROMO_URL))))

if __name__ == "__main__":
    app.run(**RUN_CONF)
